#!/usr/bin/env python
######################################################################
#
#   Module:  tiler gui
#   Version: $Id: tiler.py,v 1.4 2003/09/15 15:27:47 jsc Exp $
#   Author:  Jeffrey Clement <jclement@bluesine.com>
#   Targets: Win32, Unix
#   Web:     http://jclement.ca/
#
# GUI for Tiler application written using PythonCard.   
#
# -------------------------------------------------------------------
#
# $Log: tiler.py,v $
# Revision 1.4  2003/09/15 15:27:47  jsc
# Can do 1x1
#
# Revision 1.3  2003/09/03 17:11:05  jsc
# Fixed some bugs for Windows
#
# Revision 1.2  2003/09/03 16:53:15  jsc
# Fixes to documentation
#
# -------------------------------------------------------------------
#
# Copyright (c) 2003, Jeffrey Clement All rights reserved. 
#
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions 
# are met: 
#
# * Redistributions of source code must retain the above copyright notice, 
#   this list of conditions and the following disclaimer. 
# * Redistributions in binary form must reproduce the above copyright 
#   notice, this list of conditions and the following disclaimer in the 
#   documentation and/or other materials provided with the distribution. 
# * Neither the name of the Bluesine nor the names of its contributors 
#   may be used to endorse or promote products derived from this software 
#   without specific prior written permission. 
#
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS 
# "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED 
# TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
# PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR 
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
# EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
# PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR 
# PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
# LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING 
# NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS 
# SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
#
##########################################################################
import os
import libtile
import Image
from PythonCardPrototype import model, dialog

#py2exe hints
from PythonCardPrototype.components import statictext
from PythonCardPrototype.components import staticline
from PythonCardPrototype.components import spinner
from PythonCardPrototype.components import button
from PythonCardPrototype.components import choice
from PythonCardPrototype.components import textfield
import JpegImagePlugin
import PngImagePlugin
import BmpImagePlugin
import GifImagePlugin


from wxPython.wx import *

class Ratio:
    def __init__(self, h, w, name=None):
        self.h = h
        self.w = w
        self.name = name
    def __str__(self):
        if self.name:
            return '%sx%s - %s' % (self.h,self.w,self.name)
        return '%sx%s' % (self.h,self.w)
    def ratio(self):
        return 1.0 * self.h / self.w

class Tiler(model.Background):
    def on_openBackground(self, evt):
        # setup height spin box
        self.components.spTileHeight.value = 2  
        self.components.spTileHeight.min = 1
        self.components.spTileHeight.max = 10
        # setup width spin box
        self.components.spTileWidth.value = 2
        self.components.spTileWidth.min = 1
        self.components.spTileWidth.max = 10
        # setup width spin box
        self.components.spBorderWidth.value = 5
        self.components.spBorderWidth.min = 0
        self.components.spBorderWidth.max = 100
        # setup width spin box
        self.components.spPaddingWidth.value = 5
        self.components.spPaddingWidth.min = 0
        self.components.spPaddingWidth.max = 100
        # colors
        self.setColor(self.components.btBorderColor, (255,255,255))
        self.setColor(self.components.btBackgroundColor, (0,0,0))
        # aspect ratios
        self.ratios = []
        self.ratios.append(Ratio(4,6))
        self.ratios.append(Ratio(6,4))
        self.ratios.append(Ratio(5,7))
        self.ratios.append(Ratio(7,5))
        self.ratios.append(Ratio(8,10))
        self.ratios.append(Ratio(10,8))
        a = self.components.chAspectRatio
        for r in self.ratios:
            a.append(str(r))
        a.Select(0)
    def setColor(self, ctl, bg):
        fg=(255-bg[0], 255-bg[1], 255-bg[2])
        ctl.backgroundColor = bg
        ctl.foregroundColor = fg
    def on_mnuExit_select(self, evt):
        self.Close()
    def on_menuAbout_select(self, evt):
        self.msg('''Tiler by Jeffrey Clement
http://jclement.ca/software/tiler/

Copyright (C) 2003, Jeffrey Clement
This is free software; see the source for copying conditions.  There is NO
warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.

See the included LICENSE.BSD for details!
''')

    def on_spTileHeight_textUpdate(self, evt):
        s = self.components.spTileHeight
        # make sure it is numeric
        try: 
            int(s.value)
        except: 
            s.value=s.min
            return
        # it's in range right?
        if s.value < s.min or s.value > s.max: s.value = s.min
    def on_spTileWidth_textUpdate(self, evt):
        s = self.components.spTileWidth
        # make sure it is numeric
        try: 
            int(s.value)
        except: 
            s.value=s.min
            return
        # it's in range right?
        if s.value < s.min or s.value > s.max: s.value = s.min
    def on_txSourceFile_textUpdate(self, evt):
        dc = self.components.txDestFile
        fn = self.components.txSourceFile.text
        fn,ext = os.path.splitext(fn)
        dc.text = '%s-tiled%s' % (fn, ext)
        
    def on_btBrowse_mouseClick(self, evt):
        wildcard = "JPG files (*.jpg;*.jpeg)|*.jpeg;*.JPG;*.JPEG;*.jpg|GIF files (*.gif)|*.GIF;*.gif|All Files (*.*)|*.*"
        result = dialog.fileDialog(self, 'Open', '', '', wildcard )
        if result['accepted'] >= 1:
            self.components.txSourceFile.text = result['paths'][0]        
    def msg(self, m):
        dialog.alertDialog(self, str(m), 'Tiler!')
    def on_btTile_mouseClick(self, evt):
        tileDimensions = (int(self.components.spTileWidth.value),
                          int(self.components.spTileHeight.value))
        srcFile = self.components.txSourceFile.text
        destFile = self.components.txDestFile.text
        borderColor = self.components.btBorderColor.backgroundColor.asTuple()
        backgroundColor = self.components.btBackgroundColor.backgroundColor.asTuple()
        aspectRatio = self.ratios[self.components.chAspectRatio.GetSelection()].ratio()
        paddingWidth = self.components.spPaddingWidth.value / 100.0
        borderWidth = self.components.spBorderWidth.value / 100.0

        w=wxBusyCursor()
        if not os.path.isfile(srcFile):
            self.msg('Source file does not exist!')
            return
        try:
            src = Image.open(srcFile)
            dest = libtile.tile(
                        dimensions=tileDimensions,
                        src=src,
                        hwRatio=aspectRatio,
                        paddingPercent=borderWidth,
                        borderBg=borderColor,
                        pageBg=backgroundColor,
                        outsideBorderPercent=paddingWidth)
            dest.save(destFile)
            self.msg('Operation complete!')
        except Exception, e:
            self.msg(e)
            return
            
    def on_btBorderColor_mouseClick(self, evt):
        result = dialog.colorDialog(self)
        if result['accepted']:
            self.setColor(self.components.btBorderColor, result['color'])
    def on_btBackgroundColor_mouseClick(self, evt):
        result = dialog.colorDialog(self)
        if result['accepted']:
            self.setColor(self.components.btBackgroundColor, result['color'])

if __name__ == '__main__':
    app = model.PythonCardApp(Tiler)
    app.MainLoop()
